// mozileDetection.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/*******

Mozile Detection Javascript
Mozile version 0.6.16

Created by James A. Overton
Created on 2004-03-24

Last revised by James A. Overton
Last revised on 2004-11-16

HISTORY:
	Some code in this file was first found in mozilehtmltb.js, while the 
majority was cut out of mozileTB.js and separated into different files.


DESCRIPTION:
	The code here is responsible for detecting when Mozile should be active.
It gets called a lot, so mozileDetect() has to be very fast.

*********/

function mozileGetCurrentTarget() {
	return mozileTarget;
}

function mozileIsContentEditable(nodeToCheck) {
		while(nodeToCheck)
		{
			// find a non element node - return: probably DOCUMENT_NODE reached!
			if(nodeToCheck.nodeType != Node.ELEMENT_NODE)
				return 0;
			// if has a content editable attribute then sync it with mozusermodify
			if(nodeToCheck.getAttribute("contentEditable"))
			{
				if(nodeToCheck.getAttribute("contentEditable") == "true")
					return 1;
				return 0;
			}
			nodeToCheck = nodeToCheck.parentNode;
		}
		return 0;
}



// Mozile Detect
// Check for the mozileChromeLoaded flag.
// Check to see if the user has selected an editable area.
// If so, set the global mozileTarget and mozileSelection variables.
// If not, return 0, and don't do anything else.

function mozileNoDetect(){
 if(arguments.length==0 || arguments[0]==true){
	mozileClearCommandMenu();	mozileClearButtons();	mozile.lastContextNode=null;}
 return 0;
}

function mozileDetect() {
	try {
		// get the focused window
		var mozileCheckTarget = document.commandDispatcher.focusedWindow;
		//Ignore if in browser.
		if ( mozileCheckTarget.toString() == "[object ChromeWindow]" ) {
			return 0;	}
		//Don't think this second test is needed.
//		if(mozileCheckTarget.document.documentElement==document.documentElement) {
//			return 0;	} 
	} catch (e) {
		return mozileNoDetect();
	}
	try{
		var f = ["mozileDetect.js","mozileDetect"];
		//get the selection of the focused window
		var mozileCheckSelection = mozileCheckTarget.getSelection();

		// if there's no node, Mozile won't be able to edit
		try {
  		if ( mozileCheckSelection.anchorNode == null ) {
				throw "anchorNode==null"; }
  		if ( mozileCheckSelection.anchorNode.parentNode == null ) {
				throw "parentNode==null"; }
			var currentContextNode = mozileGetContext(mozileCheckSelection);
			if ( currentContextNode == null ) {
				throw "null return from mozileGetContext"; }
		} catch(e) {
			mozileDebug(f,3,"No node to edit: "+e);
			return mozileNoDetect(("mozileChromeLoaded" in mozileCheckTarget));
		}
		mozileDebug(f,3,"Detecting node: "+currentContextNode.tagName);

			// don't edit these nodes!			
		var ignoreList = new Array("input","textarea");
		for(var i=0; i<ignoreList.length; i++) {
			if(currentContextNode.tagName.toLowerCase()==ignoreList[i]) {
				return mozileNoDetect(("mozileChromeLoaded" in mozileCheckTarget));
			}
		}

/*
			//[MAX - this shouldn't be needed if changes to undo work correctly...
			// for x/html documents you would need to detect the <body> element as well...]
			// 
			// undo/redo can change the selection to the documentElement, so ignore a sudden change
		if( currentContextNode == mozileCheckTarget.document.documentElement ) {
			mozileDebug(f,0,"The current node is the document element");
			return 1;
		}*/

			// is it the same (editable) node?
			//This is a short circuit to save executing the code below...
		try{
			if(mozile.lastContextNode === currentContextNode) {
				mozileDebug(f,3,"We're still inside the same editable node");
				return 1;
			}
		} catch(e) { 
			throw e;
		}

		// Check if the selection is editable. Check CSS first, it's fast.
		var CSSenabled = mozileCheckTarget.document.defaultView.getComputedStyle(currentContextNode, null).MozUserModify;
		mozileDebug(f,3,"CSS Enabled? "+CSSenabled);
		if(CSSenabled!="read-write") {
		  // if no CSS check for content editable. (only check if necessary, since it's slow)
			var CEenabled = mozileIsContentEditable(currentContextNode);
			mozileDebug(f,3,"CE Enabled? "+CEenabled);
			if(CEenabled!=1) { //Selection is not editable.
				return mozileNoDetect(("mozileChromeLoaded" in mozileCheckTarget));
			}
		}

			// if mozileChromeLoaded is false, inject the code
		var mozileInjected=0;
		if( !("mozileChromeLoaded" in mozileCheckTarget) || mozileCheckTarget.mozileChromeLoaded != "true") {
			mozileInject(mozileCheckTarget);
			mozileInjected=1;
		}

		//Set global variables	
		mozileTarget = mozileCheckTarget;
		mozileSelection = mozileCheckSelection;
		mozile.lastContextNode = currentContextNode;
		try {
			mozileUpdateCommandMenu();
			mozileUpdateButtons();
			if (mozileInjected) {
				mozileUpdateUndoButtons();
			}
		}
		catch(e) {
			mozileDebug(f,1,"Exception while updating: "+e);
		}
		mozileDebug(f,3,"Detect done.");
		return 1;	// done
	}
	catch(e) { 
		mozileDebug(f,0,"Detect Died: "+e);
		return mozileNoDetect();
	}
}
